@@ -1,21 +1,9 @@
 import numpy as np
 import pandas as pd
-import statsmodels.api as sm
 from datetime import timedelta, datetime
-from sklearn.metrics import mean_absolute_error
 from fbprophet import Prophet
 from math import sqrt
 
-### reference this for future improvements
-#DEFAULT_INTERVAL = timedelta(minutes=10)
-#DEFAULT_PERIODS = 6 * 48  # 10 mins * 6 * 48 = 2 days
-#DEFAULT_MODEL_OPTIONS = {"type": "areg"}
-# df in, dictionary out
-# def forecast(
-#     df, interval=DEFAULT_INTERVAL, periods=DEFAULT_PERIODS, model_options=None
-# ):
-
-
 ### clean functions
 def dict_to_df(dict_input):
     df = pd.DataFrame(dict_input)
@@ -23,14 +11,9 @@ def dict_to_df(dict_input):
 
 def format_delta(df):
     df.read_time = pd.to_datetime(df.read_time) 
-    delta = df.volume.diff()
-    delta = delta.rename('delta')
-    df = pd.concat([df, delta], axis = 1)
-    cols = [0,1,2,6]
-    df = df.drop(df.columns[cols], axis = 1)
-    df = pd.DataFrame(df, columns = ['read_time', 'delta'])
-    df.columns = ['ds','y']
-    df.y = df.y.clip(lower = 0, upper = 250)    
+    df['y'] = df.volume.diff() * -1
+    df = df[['read_time', 'y']].rename(columns={'read_time': 'ds'})
+    df.y = df.y.clip(lower = 0, upper = 250)
     return(df)
 
 def half_hour(df):
@@ -39,16 +22,14 @@ def half_hour(df):
     df.drop(df.tail(1).index, inplace = True)
     return(df)
 
-# this is the function that should be called to clean the data
 def clean(dict_input):
+    dict_input = sorted(dict_input, key = lambda x: x['read_time'])
     df = dict_to_df(dict_input)
     df = format_delta(df)
     df = half_hour(df)
     return(df)
 
-
-
-# forecast near functions
+# short term forecasts
 def gen_future(df, periods = 48):
     start_0 = df.iloc[-1, 0]  
     future = pd.date_range(start=start_0, freq="30min", periods=periods) + pd.Timedelta(minutes = 30)
@@ -57,20 +38,24 @@ def gen_future(df, periods = 48):
     return(future)
 
 def fbp(df, future):
-    m = Prophet(changepoint_prior_scale=0.5, 
+    m = Prophet(changepoint_prior_scale=0.05, 
+                daily_seasonality = True,
                 weekly_seasonality = True, 
-                yearly_seasonality = False).add_seasonality(name = 'custom',
-                                                            period = 48, 
-                                                            fourier_order = 8, 
-                                                            prior_scale=0.01)
+                yearly_seasonality = False)
     mod = m.fit(df)
     yhat = m.predict(future)
     out = yhat[['ds','yhat_lower', 'yhat_upper', 'yhat']]
 
     return(out)
 
-# this is the function that should be called to make the forecast
-def forecast_near(df, length = 48, output = 'dict'):
+def forecast_near(df,
+                  length = 48,
+                  holidays_low = [],
+                  low_multi = 1,
+                  holidays_high = [],
+                  high_multi = 1,
+                  output = 'dict'
+                 ):
     
     status_dict = {
         0: 'Forecast Failed: not enough data were provided.',
@@ -86,25 +71,10 @@ def forecast_near(df, length = 48, output = 'dict'):
         status = 2
     else:
         return({'status': status_dict[0]}) # not enough data
-    
-### Backcasting for accuracy check ###  
- #   index = len(df)-144
- #   train = df[:index]
- #   test_yhat = fbp(train, test)
- #   mae = mean_absolute_error(test_yhat['yhat'], test.y) # calculate MAE
-
- #   if mae <= 150:
- #       status = 4
- #   elif mae <= 200:
- #       status = 3
- #   else:
-     #   return({'status': status_dict[1]}) # bad predictions
- #       status = 1
         
     future = gen_future(df, periods = length)
     out = fbp(df, future)
-    
-    # clean up output
+ 
     out.rename(columns = {                  
                       'yhat_lower':'lower',
                       'yhat_upper':'upper'
@@ -112,7 +82,28 @@ def forecast_near(df, length = 48, output = 'dict'):
         
     for field in ["yhat", "lower", "upper"]:
         out.loc[out[field] < 0, field] = 0
+    
+    # holidays
+    if holidays_low:
+        out['date'] = out.ds.dt.date
+        out.date = pd.to_datetime(out.date)
+        holidays_low = pd.to_datetime(pd.Series(holidays_low))
 
+        for field in ["yhat", "lower", "upper"]:
+            out.loc[out['date'].isin(holidays_low), field] *= low_multi
+        
+        del out['date']
+   
+    if holidays_high:
+        out['date'] = out.ds.dt.date
+        out.date = pd.to_datetime(out.date)
+        holidays_high = pd.to_datetime(pd.Series(holidays_high))
+
+        for field in ["yhat", "lower", "upper"]:
+            out.loc[out['date'].isin(holidays_high), field] *= high_multi
+        
+        del out['date']
+        
     # return desired output (dict or df)
     if output == 'df':
         return(out)
@@ -125,8 +116,7 @@ def forecast_near(df, length = 48, output = 'dict'):
             )
 
 
-
-### forecast far functions
+# long term forecasts
 def aggregate_to_daily(df):
     df = df.set_index('ds').groupby(pd.Grouper(freq='D')).sum()
     df.reset_index(inplace = True)
@@ -145,8 +135,14 @@ def add_dow(df): # this just adds labels for mon - sun (0-6)
         day_of_week[i] = datetime.weekday(df.ds[i])
     return(pd.concat([day_of_week, df], axis = 1))
 
-# this is the function that should be called to make the forecast
-def forecast_far(df, length = 30, output = 'dict'):
+# forecast_far is the actual function to call for a full prediction
+def forecast_far(df,
+                 length = 30,
+                 holidays_low = [],
+                 low_multi = 1,
+                 holidays_high = [],
+                 high_multi = 1,
+                 output = 'dict'):
     
     status_dict = {
         0: 'Forecast Failed: Not enough data were provided.',
@@ -226,6 +222,27 @@ def forecast_far(df, length = 30, output = 'dict'):
     for field in ["yhat", "lower", "upper"]:
         out.loc[out[field] < 0, field] = 0
     
+    # holiday multipliers
+    if holidays_low:
+        out['date'] = out.ds.dt.date
+        out.date = pd.to_datetime(out.date)
+        holidays_low = pd.to_datetime(pd.Series(holidays_low))
+
+        for field in ["yhat", "lower", "upper"]:
+            out.loc[out['date'].isin(holidays_low), field] *= low_multi
+        
+        del out['date']
+   
+    if holidays_high:
+        out['date'] = out.ds.dt.date
+        out.date = pd.to_datetime(out.date)
+        holidays_high = pd.to_datetime(pd.Series(holidays_high))
+
+        for field in ["yhat", "lower", "upper"]:
+            out.loc[out['date'].isin(holidays_high), field] *= high_multi
+        
+        del out['date']
+    
     # return desired output (dict or df)
     if output == 'df':
         return(out)
@@ -235,4 +252,4 @@ def forecast_far(df, length = 30, output = 'dict'):
                 "status": status_dict[status],
                 "data": out.to_dict(orient="records"),
             }
-            )
\ No newline at end of file
+            )
